@@ -1,6 +1,6 @@
 ;;; tramp-gvfs.el --- Tramp access functions for GVFS daemon
 
-;; Copyright (C) 2009-2015 Free Software Foundation, Inc.
+;; Copyright (C) 2009-2016 Free Software Foundation, Inc.
 
 ;; Author: Michael Albinus <michael.albinus@gmx.de>
 ;; Keywords: comm, processes
@@ -86,7 +86,7 @@
 
 ;; Restrictions:
 
-;; * The current GVFS implementation does not allow to write on the
+;; * The current GVFS implementation does not allow writing on the
 ;;   remote bluetooth device via OBEX.
 ;;
 ;; * Two shares of the same SMB server cannot be mounted in parallel.
@@ -407,6 +407,42 @@ Every entry is a list (NAME ADDRESS).")
 (defconst tramp-hal-interface-device "org.freedesktop.Hal.Device"
   "The device interface of the HAL daemon.")
 
+(defconst tramp-gvfs-file-attributes
+  '("type"
+    "standard::display-name"
+    ;; We don't need this one.  It is used as delimiter in case the
+    ;; display name contains spaces, which is hard to parse.
+    "standard::icon"
+    "standard::symlink-target"
+    "unix::nlink"
+    "unix::uid"
+    "owner::user"
+    "unix::gid"
+    "owner::group"
+    "time::access"
+    "time::modified"
+    "time::changed"
+    "standard::size"
+    "unix::mode"
+    "access::can-read"
+    "access::can-write"
+    "access::can-execute"
+    "unix::inode"
+    "unix::device")
+  "GVFS file attributes.")
+
+(defconst tramp-gvfs-file-attributes-with-gvfs-ls-regexp
+  (concat "[[:blank:]]"
+	  (regexp-opt tramp-gvfs-file-attributes t)
+	  "=\\([^[:blank:]]+\\)")
+  "Regexp to parse GVFS file attributes with `gvfs-ls'.")
+
+(defconst tramp-gvfs-file-attributes-with-gvfs-info-regexp
+  (concat "^[[:blank:]]*"
+	  (regexp-opt tramp-gvfs-file-attributes t)
+	  ":[[:blank:]]+\\(.*\\)$")
+  "Regexp to parse GVFS file attributes with `gvfs-info'.")
+
 
 ;; New handlers should be added here.
 (defconst tramp-gvfs-file-name-handler-alist
@@ -422,7 +458,6 @@ Every entry is a list (NAME ADDRESS).")
     (directory-files . tramp-handle-directory-files)
     (directory-files-and-attributes
      . tramp-handle-directory-files-and-attributes)
-    (dired-call-process . ignore)
     (dired-compress-file . ignore)
     (dired-uncache . tramp-handle-dired-uncache)
     (expand-file-name . tramp-gvfs-handle-expand-file-name)
@@ -474,7 +509,7 @@ Every entry is a list (NAME ADDRESS).")
     (shell-command . ignore)
     (start-file-process . ignore)
     (substitute-in-file-name . tramp-handle-substitute-in-file-name)
-    (unhandled-file-name-directory . tramp-handle-unhandled-file-name-directory)
+    (unhandled-file-name-directory . ignore)
     (vc-registered . ignore)
     (verify-visited-file-modtime . tramp-handle-verify-visited-file-modtime)
     (write-region . tramp-gvfs-handle-write-region))
@@ -562,8 +597,7 @@ will be traced by Tramp with trace level 6."
 
 (put 'with-tramp-dbus-call-method 'lisp-indent-function 2)
 (put 'with-tramp-dbus-call-method 'edebug-form-spec '(form symbolp body))
-(tramp-compat-font-lock-add-keywords
- 'emacs-lisp-mode '("\\<with-tramp-dbus-call-method\\>"))
+(font-lock-add-keywords 'emacs-lisp-mode '("\\<with-tramp-dbus-call-method\\>"))
 
 (defvar tramp-gvfs-dbus-event-vector nil
   "Current Tramp file name to be used, as vector.
@@ -623,19 +657,19 @@ file names."
 		(and t2 (not (tramp-gvfs-file-name-p newname))))
 
 	    ;; We cannot copy or rename directly.
+	    ;; PRESERVE-EXTENDED-ATTRIBUTES has been introduced with
+	    ;; Emacs 24.1 (as PRESERVE-SELINUX-CONTEXT), and renamed
+	    ;; in Emacs 24.3.
 	    (let ((tmpfile (tramp-compat-make-temp-file filename)))
 	      (cond
 	       (preserve-extended-attributes
-		(tramp-compat-funcall
+		(funcall
 		 file-operation
 		 filename tmpfile t keep-date preserve-uid-gid
 		 preserve-extended-attributes))
-	       (preserve-uid-gid
-		(tramp-compat-funcall
-		 file-operation filename tmpfile t keep-date preserve-uid-gid))
 	       (t
-		(tramp-compat-funcall
-		 file-operation filename tmpfile t keep-date)))
+		(funcall
+		 file-operation filename tmpfile t keep-date preserve-uid-gid)))
 	      (rename-file tmpfile newname ok-if-already-exists))
 
 	  ;; Direct action.
@@ -646,7 +680,7 @@ file names."
 		 'tramp-gvfs-send-command v gvfs-operation
 		 (append
 		  (and (eq op 'copy) (or keep-date preserve-uid-gid)
-		       (list "--preserve"))
+		       '("--preserve"))
 		  (list
 		   (tramp-gvfs-url-file-name filename)
 		   (tramp-gvfs-url-file-name newname))))
@@ -693,19 +727,18 @@ file names."
     (tramp-gvfs-do-copy-or-rename-file
      'copy filename newname ok-if-already-exists keep-date
      preserve-uid-gid preserve-extended-attributes))
-   ;; Compat section.
+   ;; Compat section.  PRESERVE-EXTENDED-ATTRIBUTES has been
+   ;; introduced with Emacs 24.1 (as PRESERVE-SELINUX-CONTEXT), and
+   ;; renamed in Emacs 24.3.
    (preserve-extended-attributes
     (tramp-run-real-handler
      'copy-file
      (list filename newname ok-if-already-exists keep-date
 	   preserve-uid-gid preserve-extended-attributes)))
-   (preserve-uid-gid
-    (tramp-run-real-handler
-     'copy-file
-     (list filename newname ok-if-already-exists keep-date preserve-uid-gid)))
    (t
     (tramp-run-real-handler
-     'copy-file (list filename newname ok-if-already-exists keep-date)))))
+     'copy-file
+     (list filename newname ok-if-already-exists keep-date preserve-uid-gid)))))
 
 (defun tramp-gvfs-handle-delete-directory (directory &optional recursive trash)
   "Like `delete-directory' for Tramp files."
@@ -787,127 +820,185 @@ file names."
        (tramp-run-real-handler
 	'expand-file-name (list localname))))))
 
-(defun tramp-gvfs-handle-file-attributes (filename &optional id-format)
-  "Like `file-attributes' for Tramp files."
-  (unless id-format (setq id-format 'integer))
+(defun tramp-gvfs-get-directory-attributes (directory)
+  "Return GVFS attributes association list of all files in DIRECTORY."
   (ignore-errors
     ;; Don't modify `last-coding-system-used' by accident.
     (let ((last-coding-system-used last-coding-system-used)
-	  (process-environment (cons "LC_MESSAGES=C" process-environment))
-	  dirp res-symlink-target res-numlinks res-uid res-gid res-access
-	  res-mod res-change res-size res-filemodes res-inode res-device)
+	  result)
+      (with-parsed-tramp-file-name directory nil
+	(with-tramp-file-property v localname "directory-gvfs-attributes"
+	  (tramp-message v 5 "directory gvfs attributes: %s" localname)
+	  ;; Send command.
+	  (tramp-gvfs-send-command
+	   v "gvfs-ls" "-h" "-n" "-a"
+	   (mapconcat 'identity tramp-gvfs-file-attributes ",")
+	   (tramp-gvfs-url-file-name directory))
+	  ;; Parse output ...
+	  (with-current-buffer (tramp-get-connection-buffer v)
+	    (goto-char (point-min))
+	    (while (re-search-forward
+		    (concat "^\\(.+\\)[[:blank:]]"
+			    "\\([[:digit:]]+\\)[[:blank:]]"
+			    "(\\(.+\\))[[:blank:]]"
+			    "standard::display-name=\\(.+\\)[[:blank:]]"
+			    "standard::icon=")
+		    (point-at-eol) t)
+	      (let ((item (list (cons "standard::display-name" (match-string 4))
+				(cons "type" (match-string 3))
+				(cons "standard::size" (match-string 2))
+				(match-string 1))))
+		(while (re-search-forward
+			tramp-gvfs-file-attributes-with-gvfs-ls-regexp
+			(point-at-eol) t)
+		  (push (cons (match-string 1) (match-string 2)) item))
+		(push (nreverse item) result))
+	      (forward-line)))
+	  result)))))
+
+(defun tramp-gvfs-get-root-attributes (filename)
+  "Return GVFS attributes association list of FILENAME."
+  (ignore-errors
+    ;; Don't modify `last-coding-system-used' by accident.
+    (let ((last-coding-system-used last-coding-system-used)
+	  result)
       (with-parsed-tramp-file-name filename nil
-	(with-tramp-file-property
-	    v localname (format "file-attributes-%s" id-format)
-	  (tramp-message v 5 "file attributes: %s" localname)
+	(with-tramp-file-property v localname "file-gvfs-attributes"
+	  (tramp-message v 5 "file gvfs attributes: %s" localname)
+	  ;; Send command.
 	  (tramp-gvfs-send-command
 	   v "gvfs-info" (tramp-gvfs-url-file-name filename))
 	  ;; Parse output ...
 	  (with-current-buffer (tramp-get-connection-buffer v)
 	    (goto-char (point-min))
-	    (when (re-search-forward "attributes:" nil t)
-	      ;; ... directory or symlink
-	      (goto-char (point-min))
-	      (setq dirp (if (re-search-forward "type: directory" nil t) t))
-	      (goto-char (point-min))
-	      (setq res-symlink-target
-		    (if (re-search-forward
-			 "standard::symlink-target: \\(.+\\)$" nil t)
-			(match-string 1)))
-	      ;; ... number links
-	      (goto-char (point-min))
-	      (setq res-numlinks
-		    (if (re-search-forward "unix::nlink: \\([0-9]+\\)" nil t)
-			(string-to-number (match-string 1)) 0))
-	      ;; ... uid and gid
-	      (goto-char (point-min))
-	      (setq res-uid
-		    (if (eq id-format 'integer)
-			(if (re-search-forward "unix::uid: \\([0-9]+\\)" nil t)
-			    (string-to-number (match-string 1))
-			  -1)
-		      (if (re-search-forward "owner::user: \\(.+\\)$" nil t)
-			  (match-string 1)
-			"UNKNOWN")))
-	      (setq res-gid
-		    (if (eq id-format 'integer)
-			(if (re-search-forward "unix::gid: \\([0-9]+\\)" nil t)
-			    (string-to-number (match-string 1))
-			  -1)
-		      (if (re-search-forward "owner::group: \\(.+\\)$" nil t)
-			  (match-string 1)
-			"UNKNOWN")))
-	      ;; ... last access, modification and change time
-	      (goto-char (point-min))
-	      (setq res-access
-		    (if (re-search-forward "time::access: \\([0-9]+\\)" nil t)
-			(seconds-to-time (string-to-number (match-string 1)))
-		      '(0 0)))
-	      (goto-char (point-min))
-	      (setq res-mod
-		    (if (re-search-forward "time::modified: \\([0-9]+\\)" nil t)
-			(seconds-to-time (string-to-number (match-string 1)))
-		      '(0 0)))
-	      (goto-char (point-min))
-	      (setq res-change
-		    (if (re-search-forward "time::changed: \\([0-9]+\\)" nil t)
-			(seconds-to-time (string-to-number (match-string 1)))
-		      '(0 0)))
-	      ;; ... size
-	      (goto-char (point-min))
-	      (setq res-size
-		    (if (re-search-forward "standard::size: \\([0-9]+\\)" nil t)
-			(string-to-number (match-string 1)) 0))
-	      ;; ... file mode flags
-	      (goto-char (point-min))
-	      (setq res-filemodes
-		    (if (re-search-forward "unix::mode: \\([0-9]+\\)" nil t)
-			(tramp-file-mode-from-int
-			 (string-to-number (match-string 1)))
-		      (if dirp "drwx------" "-rwx------")))
-	      ;; ... inode and device
-	      (goto-char (point-min))
-	      (setq res-inode
-		    (if (re-search-forward "unix::inode: \\([0-9]+\\)" nil t)
-			(string-to-number (match-string 1))
-		      (tramp-get-inode v)))
-	      (goto-char (point-min))
-	      (setq res-device
-		    (if (re-search-forward "unix::device: \\([0-9]+\\)" nil t)
-			(string-to-number (match-string 1))
-		      (tramp-get-device v)))
+	    (while (re-search-forward
+		    tramp-gvfs-file-attributes-with-gvfs-info-regexp nil t)
+	      (push (cons (match-string 1) (match-string 2)) result))
+	    result))))))
 
-	      ;; Return data gathered.
-	      (list
-	       ;; 0. t for directory, string (name linked to) for
-	       ;; symbolic link, or nil.
-	       (or dirp res-symlink-target)
-	       ;; 1. Number of links to file.
-	       res-numlinks
-	       ;; 2. File uid.
-	       res-uid
-	       ;; 3. File gid.
-	       res-gid
-	       ;; 4. Last access time, as a list of integers.
-	       ;; 5. Last modification time, likewise.
-	       ;; 6. Last status change time, likewise.
-	       res-access res-mod res-change
-	       ;; 7. Size in bytes (-1, if number is out of range).
-	       res-size
-	       ;; 8. File modes.
-	       res-filemodes
-	       ;; 9. t if file's gid would change if file were deleted
-	       ;; and recreated.
-	       nil
-	       ;; 10. Inode number.
-	       res-inode
-	       ;; 11. Device number.
-	       res-device
-	       ))))))))
+(defun tramp-gvfs-get-file-attributes (filename)
+  "Return GVFS attributes association list of FILENAME."
+  (setq filename (directory-file-name (expand-file-name filename)))
+  (with-parsed-tramp-file-name filename nil
+    (if (or
+	 (and (string-match "^\\(afp\\|smb\\)$" method)
+	      (string-match "^/?\\([^/]+\\)$" localname))
+	 (string-equal localname "/"))
+	(tramp-gvfs-get-root-attributes filename)
+      (assoc
+       (file-name-nondirectory filename)
+       (tramp-gvfs-get-directory-attributes (file-name-directory filename))))))
+
+(defun tramp-gvfs-handle-file-attributes (filename &optional id-format)
+  "Like `file-attributes' for Tramp files."
+  (unless id-format (setq id-format 'integer))
+  (ignore-errors
+    (let ((attributes (tramp-gvfs-get-file-attributes filename))
+	  dirp res-symlink-target res-numlinks res-uid res-gid res-access
+	  res-mod res-change res-size res-filemodes res-inode res-device)
+      (when attributes
+	;; ... directory or symlink
+	(setq dirp (if (equal "directory" (cdr (assoc "type" attributes))) t))
+	(setq res-symlink-target
+	      (cdr (assoc "standard::symlink-target" attributes)))
+	;; ... number links
+	(setq res-numlinks
+	      (string-to-number
+	       (or (cdr (assoc "unix::nlink" attributes)) "0")))
+	;; ... uid and gid
+	(setq res-uid
+	      (if (eq id-format 'integer)
+		  (string-to-number
+		   (or (cdr (assoc "unix::uid" attributes))
+		       (format "%s" tramp-unknown-id-integer)))
+		(or (cdr (assoc "owner::user" attributes))
+		    (cdr (assoc "unix::uid" attributes))
+		    tramp-unknown-id-string)))
+	(setq res-gid
+	      (if (eq id-format 'integer)
+		  (string-to-number
+		   (or (cdr (assoc "unix::gid" attributes))
+		       (format "%s" tramp-unknown-id-integer)))
+		(or (cdr (assoc "owner::group" attributes))
+		    (cdr (assoc "unix::gid" attributes))
+		    tramp-unknown-id-string)))
+	;; ... last access, modification and change time
+	(setq res-access
+	      (seconds-to-time
+	       (string-to-number
+		(or (cdr (assoc "time::access" attributes)) "0"))))
+	(setq res-mod
+	      (seconds-to-time
+	       (string-to-number
+		(or (cdr (assoc "time::modified" attributes)) "0"))))
+	(setq res-change
+	      (seconds-to-time
+	       (string-to-number
+		(or (cdr (assoc "time::changed" attributes)) "0"))))
+	;; ... size
+	(setq res-size
+	      (string-to-number
+	       (or (cdr (assoc "standard::size" attributes)) "0")))
+	;; ... file mode flags
+	(setq res-filemodes
+	      (let ((n (cdr (assoc "unix::mode" attributes))))
+		(if n
+		    (tramp-file-mode-from-int (string-to-number n))
+		  (format
+		   "%s%s%s%s------"
+		   (if dirp "d" "-")
+		   (if (equal (cdr (assoc "access::can-read" attributes))
+			      "FALSE")
+		       "-" "r")
+		   (if (equal (cdr (assoc "access::can-write" attributes))
+			      "FALSE")
+		       "-" "w")
+		   (if (equal (cdr (assoc "access::can-execute" attributes))
+			      "FALSE")
+		       "-" "x")))))
+	;; ... inode and device
+	(setq res-inode
+	      (let ((n (cdr (assoc "unix::inode" attributes))))
+		(if n
+		    (string-to-number n)
+		  (tramp-get-inode (tramp-dissect-file-name filename)))))
+	(setq res-device
+	      (let ((n (cdr (assoc "unix::device" attributes))))
+		(if n
+		    (string-to-number n)
+		  (tramp-get-device (tramp-dissect-file-name filename)))))
+
+	;; Return data gathered.
+	(list
+	 ;; 0. t for directory, string (name linked to) for
+	 ;; symbolic link, or nil.
+	 (or dirp res-symlink-target)
+	 ;; 1. Number of links to file.
+	 res-numlinks
+	 ;; 2. File uid.
+	 res-uid
+	 ;; 3. File gid.
+	 res-gid
+	 ;; 4. Last access time, as a list of integers.
+	 ;; 5. Last modification time, likewise.
+	 ;; 6. Last status change time, likewise.
+	 res-access res-mod res-change
+	 ;; 7. Size in bytes (-1, if number is out of range).
+	 res-size
+	 ;; 8. File modes.
+	 res-filemodes
+	 ;; 9. t if file's gid would change if file were deleted
+	 ;; and recreated.
+	 nil
+	 ;; 10. Inode number.
+	 res-inode
+	 ;; 11. Device number.
+	 res-device
+	 )))))
 
 (defun tramp-gvfs-handle-file-directory-p (filename)
   "Like `file-directory-p' for Tramp files."
-  (eq t (car (file-attributes filename))))
+  (eq t (car (file-attributes (file-truename filename)))))
 
 (defun tramp-gvfs-handle-file-executable-p (filename)
   "Like `file-executable-p' for Tramp files."
@@ -923,79 +1014,27 @@ file names."
 	(tramp-error
 	 v 'file-error
 	 "Cannot make local copy of non-existing file `%s'" filename))
-      (copy-file filename tmpfile t t)
+      (copy-file filename tmpfile 'ok-if-already-exists 'keep-time)
       tmpfile)))
 
 (defun tramp-gvfs-handle-file-name-all-completions (filename directory)
   "Like `file-name-all-completions' for Tramp files."
   (unless (save-match-data (string-match "/" filename))
-    (with-parsed-tramp-file-name (expand-file-name directory) nil
-
-      (all-completions
-       filename
-       (mapcar
-	'list
-        (or
-	 ;; Try cache entries for filename, filename with last
-	 ;; character removed, filename with last two characters
-	 ;; removed, ..., and finally the empty string - all
-	 ;; concatenated to the local directory name.
-         (let ((remote-file-name-inhibit-cache
-		(or remote-file-name-inhibit-cache
-		    tramp-completion-reread-directory-timeout)))
-
-	   ;; This is inefficient for very long filenames, pity
-	   ;; `reduce' is not available...
-	   (car
-	    (apply
-	     'append
-	     (mapcar
-	      (lambda (x)
-		(let ((cache-hit
-		       (tramp-get-file-property
-			v
-			(concat localname (substring filename 0 x))
-			"file-name-all-completions"
-			nil)))
-		  (when cache-hit (list cache-hit))))
-	      ;; We cannot use a length of 0, because file properties
-	      ;; for "foo" and "foo/" are identical.
-	      (tramp-compat-number-sequence (length filename) 1 -1)))))
-
-         ;; Cache expired or no matching cache entry found so we need
-         ;; to perform a remote operation.
-         (let ((result '("." ".."))
+    (all-completions
+     filename
+     (with-parsed-tramp-file-name (expand-file-name directory) nil
+       (with-tramp-file-property v localname "file-name-all-completions"
+         (let ((result '("./" "../"))
 	       entry)
            ;; Get a list of directories and files.
-	   (tramp-gvfs-send-command
-	    v "gvfs-ls" "-h" (tramp-gvfs-url-file-name directory))
-
-	   ;; Now grab the output.
-           (with-temp-buffer
-	     (insert-buffer-substring (tramp-get-connection-buffer v))
-             (goto-char (point-max))
-             (while (zerop (forward-line -1))
-	       (setq entry (buffer-substring (point) (point-at-eol)))
-	       (when (string-match filename entry)
-		 (if (file-directory-p (expand-file-name entry directory))
-		     (push (concat entry "/") result)
-		   (push entry result)))))
-
-           ;; Because the remote op went through OK we know the
-           ;; directory we `cd'-ed to exists.
-           (tramp-set-file-property v localname "file-exists-p" t)
-
-           ;; Because the remote op went through OK we know every
-           ;; file listed by `ls' exists.
-           (mapc (lambda (entry)
-		   (tramp-set-file-property
-		    v (concat localname entry) "file-exists-p" t))
-		 result)
-
-           ;; Store result in the cache.
-           (tramp-set-file-property
-            v (concat localname filename)
-	    "file-name-all-completions" result))))))))
+	   (dolist (item (tramp-gvfs-get-directory-attributes directory) result)
+	     (setq entry
+		   (or ;; Use display-name if available (google-drive).
+		    ;(cdr (assoc "standard::display-name" item))
+		    (car item)))
+	     (if (string-equal (cdr (assoc "type" item)) "directory")
+		 (push (file-name-as-directory entry) result)
+	       (push entry result)))))))))
 
 (defun tramp-gvfs-handle-file-notify-add-watch (file-name flags _callback)
   "Like `file-notify-add-watch' for Tramp files."
@@ -1024,9 +1063,9 @@ file names."
 	(tramp-message
 	 v 6 "Run `%s', %S" (mapconcat 'identity (process-command p) " ") p)
 	(tramp-set-connection-property p "vector" v)
-	(tramp-compat-process-put p 'events events)
-	(tramp-compat-process-put p 'watch-name localname)
-	(tramp-compat-set-process-query-on-exit-flag p nil)
+	(process-put p 'events events)
+	(process-put p 'watch-name localname)
+	(set-process-query-on-exit-flag p nil)
 	(set-process-filter p 'tramp-gvfs-monitor-file-process-filter)
 	;; There might be an error if the monitor is not supported.
 	;; Give the filter a chance to read the output.
@@ -1039,7 +1078,7 @@ file names."
 (defun tramp-gvfs-monitor-file-process-filter (proc string)
   "Read output from \"gvfs-monitor-file\" and add corresponding \
 file-notify events."
-  (let* ((rest-string (tramp-compat-process-get proc 'rest-string))
+  (let* ((rest-string (process-get proc 'rest-string))
 	 (dd (with-current-buffer (process-buffer proc) default-directory))
 	 (ddu (regexp-quote (tramp-gvfs-url-file-name dd))))
     (when rest-string
@@ -1047,7 +1086,7 @@ file-notify events."
     (tramp-message proc 6 "%S\n%s" proc string)
     (setq string (concat rest-string string)
 	  ;; Attribute change is returned in unused wording.
-	  string (tramp-compat-replace-regexp-in-string
+	  string (replace-regexp-in-string
 		  "ATTRIB CHANGED" "ATTRIBUTE_CHANGED" string))
     (when (string-match "Monitoring not supported" string)
       (delete-process proc))
@@ -1060,7 +1099,7 @@ file-notify events."
 	    string)
       (let ((file (match-string 1 string))
 	    (action (intern-soft
-		     (tramp-compat-replace-regexp-in-string
+		     (replace-regexp-in-string
 		      "_" "-" (downcase (match-string 2 string))))))
 	(setq string (replace-match "" nil nil string))
 	;; File names are returned as URL paths.  We must convert them.
@@ -1079,12 +1118,12 @@ file-notify events."
     ;; Save rest of the string.
     (when (zerop (length string)) (setq string nil))
     (when string (tramp-message proc 10 "Rest string:\n%s" string))
-    (tramp-compat-process-put proc 'rest-string string)))
+    (process-put proc 'rest-string string)))
 
 (defun tramp-gvfs-handle-file-readable-p (filename)
   "Like `file-readable-p' for Tramp files."
   (with-parsed-tramp-file-name filename nil
-    (with-tramp-file-property v localname "file-executable-p"
+    (with-tramp-file-property v localname "file-readable-p"
       (tramp-check-cached-permissions v ?r))))
 
 (defun tramp-gvfs-handle-file-writable-p (filename)
@@ -1125,7 +1164,8 @@ file-notify events."
   (if (or (tramp-tramp-file-p filename)
           (tramp-tramp-file-p newname))
       (tramp-gvfs-do-copy-or-rename-file
-       'rename filename newname ok-if-already-exists t t)
+       'rename filename newname ok-if-already-exists
+       'keep-date 'preserve-uid-gid)
     (tramp-run-real-handler
      'rename-file (list filename newname ok-if-already-exists))))
 
@@ -1133,8 +1173,7 @@ file-notify events."
   (start end filename &optional append visit lockname confirm)
   "Like `write-region' for Tramp files."
   (with-parsed-tramp-file-name filename nil
-    ;; XEmacs takes a coding system as the seventh argument, not `confirm'.
-    (when (and (not (featurep 'xemacs)) confirm (file-exists-p filename))
+    (when (and confirm (file-exists-p filename))
       (unless (y-or-n-p (format "File %s exists; overwrite anyway? " filename))
 	(tramp-error v 'file-error "File not overwritten")))
 
@@ -1203,8 +1242,7 @@ file-notify events."
 (defun tramp-gvfs-file-name (object-path)
   "Retrieve file name from D-Bus OBJECT-PATH."
   (dbus-unescape-from-identifier
-   (tramp-compat-replace-regexp-in-string
-    "^.*/\\([^/]+\\)$" "\\1" object-path)))
+   (replace-regexp-in-string "^.*/\\([^/]+\\)$" "\\1" object-path)))
 
 (defun tramp-bluez-address (device)
   "Return bluetooth device address from a given bluetooth DEVICE name."
@@ -1293,7 +1331,7 @@ ADDRESS can have the form \"xx:xx:xx:xx:xx:xx\" or \"[xx:xx:xx:xx:xx:xx]\"."
 	    ;; host signature.
 	    (with-temp-buffer
 	      ;; Preserve message for `progress-reporter'.
-	      (tramp-compat-with-temp-message ""
+	      (with-temp-message ""
 		(insert message)
 		(pop-to-buffer (current-buffer))
 		(setq choice (if (yes-or-no-p (concat (car choices) " ")) 0 1))
@@ -1532,8 +1570,8 @@ connection if a previous connection has died for some reason."
     (let ((p (make-network-process
 	      :name (tramp-buffer-name vec)
 	      :buffer (tramp-get-connection-buffer vec)
-	      :server t :host 'local :service t)))
-      (tramp-compat-set-process-query-on-exit-flag p nil)))
+	      :server t :host 'local :service t :noquery t)))
+      (set-process-query-on-exit-flag p nil)))
 
   (unless (tramp-gvfs-connection-mounted-p vec)
     (let* ((method (tramp-file-name-method vec))
@@ -1617,7 +1655,11 @@ connection if a previous connection has died for some reason."
 	;; is marked with the fuse-mountpoint "/".  We shall react.
 	(when (string-equal
 	       (tramp-get-file-property vec "/" "fuse-mountpoint" "") "/")
-	  (tramp-error vec 'file-error "FUSE mount denied")))))
+	  (tramp-error vec 'file-error "FUSE mount denied"))
+
+	;; Mark it as connected.
+	(tramp-set-connection-property
+	 (tramp-get-connection-process vec) "connected" t))))
 
   ;; In `tramp-check-cached-permissions', the connection properties
   ;; {uig,gid}-{integer,string} are used.  We set them to their local
@@ -1635,10 +1677,17 @@ connection if a previous connection has died for some reason."
   "Send the COMMAND with its ARGS to connection VEC.
 COMMAND is usually a command from the gvfs-* utilities.
 `call-process' is applied, and it returns t if the return code is zero."
-  (with-current-buffer (tramp-get-connection-buffer vec)
-    (tramp-gvfs-maybe-open-connection vec)
-    (erase-buffer)
-    (zerop (apply 'tramp-call-process vec command nil t nil args))))
+  (let* ((locale (tramp-get-local-locale vec))
+	 (process-environment
+	  (append
+	   `(,(format "LANG=%s" locale)
+	     ,(format "LANGUAGE=%s" locale)
+	     ,(format "LC_ALL=%s" locale))
+	   process-environment)))
+    (with-current-buffer (tramp-get-connection-buffer vec)
+      (tramp-gvfs-maybe-open-connection vec)
+      (erase-buffer)
+      (zerop (apply 'tramp-call-process vec command nil t nil args)))))
 
 
 ;; D-Bus BLUEZ functions.
@@ -1751,7 +1800,7 @@ This uses \"avahi-browse\" in case D-Bus is not enabled in Avahi."
 	    'split-string
 	    (shell-command-to-string (format "avahi-browse -trkp %s" service))
 	    "[\n\r]+" 'omit "^\\+;.*$"))))
-    (tramp-compat-delete-dups
+    (delete-dups
      (mapcar
       (lambda (x)
 	(let* ((list (split-string x ";"))
@@ -1772,35 +1821,37 @@ This uses \"avahi-browse\" in case D-Bus is not enabled in Avahi."
 
 ;; Add completion functions for AFP, DAV, DAVS, SFTP and SMB methods.
 (when tramp-gvfs-enabled
-  (zeroconf-init tramp-gvfs-zeroconf-domain)
-  (if (zeroconf-list-service-types)
-      (progn
+  ;; Suppress D-Bus error messages.
+  (let (tramp-gvfs-dbus-event-vector)
+    (zeroconf-init tramp-gvfs-zeroconf-domain)
+    (if (zeroconf-list-service-types)
+	(progn
+	  (tramp-set-completion-function
+	   "afp" '((tramp-zeroconf-parse-device-names "_afpovertcp._tcp")))
+	  (tramp-set-completion-function
+	   "dav" '((tramp-zeroconf-parse-device-names "_webdav._tcp")))
+	  (tramp-set-completion-function
+	   "davs" '((tramp-zeroconf-parse-device-names "_webdav._tcp")))
+	  (tramp-set-completion-function
+	   "sftp" '((tramp-zeroconf-parse-device-names "_ssh._tcp")
+		    (tramp-zeroconf-parse-device-names "_workstation._tcp")))
+	  (when (member "smb" tramp-gvfs-methods)
+	    (tramp-set-completion-function
+	     "smb" '((tramp-zeroconf-parse-device-names "_smb._tcp")))))
+
+      (when (executable-find "avahi-browse")
 	(tramp-set-completion-function
-	 "afp" '((tramp-zeroconf-parse-device-names "_afpovertcp._tcp")))
+	 "afp" '((tramp-gvfs-parse-device-names "_afpovertcp._tcp")))
 	(tramp-set-completion-function
-	 "dav" '((tramp-zeroconf-parse-device-names "_webdav._tcp")))
+	 "dav" '((tramp-gvfs-parse-device-names "_webdav._tcp")))
 	(tramp-set-completion-function
-	 "davs" '((tramp-zeroconf-parse-device-names "_webdav._tcp")))
+	 "davs" '((tramp-gvfs-parse-device-names "_webdav._tcp")))
 	(tramp-set-completion-function
-	 "sftp" '((tramp-zeroconf-parse-device-names "_ssh._tcp")
-		  (tramp-zeroconf-parse-device-names "_workstation._tcp")))
+	 "sftp" '((tramp-gvfs-parse-device-names "_ssh._tcp")
+		  (tramp-gvfs-parse-device-names "_workstation._tcp")))
 	(when (member "smb" tramp-gvfs-methods)
 	  (tramp-set-completion-function
-	   "smb" '((tramp-zeroconf-parse-device-names "_smb._tcp")))))
-
-    (when (executable-find "avahi-browse")
-      (tramp-set-completion-function
-       "afp" '((tramp-gvfs-parse-device-names "_afpovertcp._tcp")))
-      (tramp-set-completion-function
-       "dav" '((tramp-gvfs-parse-device-names "_webdav._tcp")))
-      (tramp-set-completion-function
-       "davs" '((tramp-gvfs-parse-device-names "_webdav._tcp")))
-      (tramp-set-completion-function
-       "sftp" '((tramp-gvfs-parse-device-names "_ssh._tcp")
-		(tramp-gvfs-parse-device-names "_workstation._tcp")))
-      (when (member "smb" tramp-gvfs-methods)
-	(tramp-set-completion-function
-	 "smb" '((tramp-gvfs-parse-device-names "_smb._tcp")))))))
+	   "smb" '((tramp-gvfs-parse-device-names "_smb._tcp"))))))))
 
 
 ;; D-Bus SYNCE functions.
