@@ -1,6 +1,6 @@
 ;;; tramp-smb.el --- Tramp access functions for SMB servers
 
-;; Copyright (C) 2002-2015 Free Software Foundation, Inc.
+;; Copyright (C) 2002-2016 Free Software Foundation, Inc.
 
 ;; Author: Michael Albinus <michael.albinus@gmx.de>
 ;; Keywords: comm, processes
@@ -129,7 +129,8 @@ call, letting the SMB client use the default one."
 	 "ERRnosuchshare"
 	 ;; Windows 4.0 (Windows NT), Windows 5.0 (Windows 2000),
 	 ;; Windows 5.1 (Windows XP), Windows 5.2 (Windows Server 2003),
-	 ;; Windows 6.0 (Windows Vista), Windows 6.1 (Windows 7).
+	 ;; Windows 6.0 (Windows Vista), Windows 6.1 (Windows 7),
+	 ;; Windows 6.3 (Windows 10).
 	 "NT_STATUS_ACCESS_DENIED"
 	 "NT_STATUS_ACCOUNT_LOCKED_OUT"
 	 "NT_STATUS_BAD_NETWORK_NAME"
@@ -224,7 +225,6 @@ See `tramp-actions-before-shell' for more info.")
     (directory-files . tramp-smb-handle-directory-files)
     (directory-files-and-attributes
      . tramp-handle-directory-files-and-attributes)
-    (dired-call-process . ignore)
     (dired-compress-file . ignore)
     (dired-uncache . tramp-handle-dired-uncache)
     (expand-file-name . tramp-smb-handle-expand-file-name)
@@ -232,10 +232,10 @@ See `tramp-actions-before-shell' for more info.")
     (file-acl . tramp-smb-handle-file-acl)
     (file-attributes . tramp-smb-handle-file-attributes)
     (file-directory-p .  tramp-smb-handle-file-directory-p)
-    ;; `file-equal-p' performed by default handler.
+    (file-file-equal-p . tramp-handle-file-equal-p)
     (file-executable-p . tramp-handle-file-exists-p)
     (file-exists-p . tramp-handle-file-exists-p)
-    ;; `file-in-directory-p' performed by default handler.
+    (file-in-directory-p . tramp-handle-file-in-directory-p)
     (file-local-copy . tramp-smb-handle-file-local-copy)
     (file-modes . tramp-handle-file-modes)
     (file-name-all-completions . tramp-smb-handle-file-name-all-completions)
@@ -247,6 +247,7 @@ See `tramp-actions-before-shell' for more info.")
     (file-newer-than-file-p . tramp-handle-file-newer-than-file-p)
     (file-notify-add-watch . tramp-handle-file-notify-add-watch)
     (file-notify-rm-watch . tramp-handle-file-notify-rm-watch)
+    (file-notify-valid-p . tramp-handle-file-notify-valid-p)
     (file-ownership-preserved-p . ignore)
     (file-readable-p . tramp-handle-file-exists-p)
     (file-regular-p . tramp-handle-file-regular-p)
@@ -275,7 +276,7 @@ See `tramp-actions-before-shell' for more info.")
     (shell-command . tramp-handle-shell-command)
     (start-file-process . tramp-smb-handle-start-file-process)
     (substitute-in-file-name . tramp-smb-handle-substitute-in-file-name)
-    (unhandled-file-name-directory . tramp-handle-unhandled-file-name-directory)
+    (unhandled-file-name-directory . ignore)
     (vc-registered . ignore)
     (verify-visited-file-modtime . tramp-handle-verify-visited-file-modtime)
     (write-region . tramp-smb-handle-write-region))
@@ -418,15 +419,14 @@ pass to the OPERATION."
 	      (unwind-protect
 		  (progn
 		    (make-directory tmpdir)
-		    (tramp-compat-copy-directory
-		     dirname tmpdir keep-date 'parents)
-		    (tramp-compat-copy-directory
+		    (copy-directory dirname tmpdir keep-date 'parents)
+		    (copy-directory
 		     (expand-file-name (file-name-nondirectory dirname) tmpdir)
 		     newname keep-date parents))
-		(tramp-compat-delete-directory tmpdir 'recursive))))
+		(delete-directory tmpdir 'recursive))))
 
 	   ;; We can copy recursively.
-	   ((or t1 t2)
+	   ((and (or t1 t2) (tramp-smb-get-cifs-capabilities v))
 	    (when (and (file-directory-p newname)
 		       (not (string-equal (file-name-nondirectory dirname)
 					  (file-name-nondirectory newname))))
@@ -447,7 +447,7 @@ pass to the OPERATION."
 		   (port      (tramp-file-name-port v))
 		   (share     (tramp-smb-get-share v))
 		   (localname (file-name-as-directory
-			       (tramp-compat-replace-regexp-in-string
+			       (replace-regexp-in-string
 				"\\\\" "/" (tramp-smb-get-localname v))))
 		   (tmpdir    (make-temp-name
 			       (expand-file-name
@@ -509,7 +509,7 @@ pass to the OPERATION."
 		      (tramp-message
 		       v 6 "%s" (mapconcat 'identity (process-command p) " "))
 		      (tramp-set-connection-property p "vector" v)
-		      (tramp-compat-set-process-query-on-exit-flag p nil)
+		      (set-process-query-on-exit-flag p nil)
 		      (tramp-process-actions p v nil tramp-smb-actions-with-tar)
 
 		      (while (memq (process-status p) '(run open))
@@ -519,7 +519,7 @@ pass to the OPERATION."
 		;; Reset the transfer process properties.
 		(tramp-set-connection-property v "process-name" nil)
 		(tramp-set-connection-property v "process-buffer" nil)
-		(when t1 (tramp-compat-delete-directory tmpdir 'recurse))))
+		(when t1 (delete-directory tmpdir 'recurse))))
 
 	    ;; Handle KEEP-DATE argument.
 	    (when keep-date
@@ -554,7 +554,8 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
       0 (format "Copying %s to %s" filename newname)
 
     (if (file-directory-p filename)
-	(tramp-compat-copy-directory filename newname keep-date t t)
+	(tramp-compat-copy-directory
+	 filename newname keep-date 'parents 'copy-contents)
 
       (let ((tmpfile (file-local-copy filename)))
 	(if tmpfile
@@ -600,7 +601,7 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
 	(mapc
 	 (lambda (file)
 	   (if (file-directory-p file)
-	       (tramp-compat-delete-directory file recursive)
+	       (delete-directory file recursive)
 	     (delete-file file)))
 	 ;; We do not want to delete "." and "..".
 	 (directory-files
@@ -648,8 +649,7 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
   (directory &optional full match nosort)
   "Like `directory-files' for Tramp files."
   (let ((result (mapcar 'directory-file-name
-			(file-name-all-completions "" directory)))
-	res)
+			(file-name-all-completions "" directory))))
     ;; Discriminate with regexp.
     (when match
       (setq result
@@ -664,9 +664,7 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
 	     result)))
     ;; Sort them if necessary.
     (unless nosort (setq result (sort result 'string-lessp)))
-    ;; Remove double entries.
-    (dolist (elt result res)
-      (add-to-list 'res elt 'append))))
+    result))
 
 (defun tramp-smb-handle-expand-file-name (name &optional dir)
   "Like `expand-file-name' for Tramp files."
@@ -731,7 +729,7 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
 	       (domain    (tramp-file-name-domain v))
 	       (port      (tramp-file-name-port v))
 	       (share     (tramp-smb-get-share v))
-	       (localname (tramp-compat-replace-regexp-in-string
+	       (localname (replace-regexp-in-string
 			   "\\\\" "/" (tramp-smb-get-localname v)))
 	       (args      (list (concat "//" real-host "/" share) "-E")))
 
@@ -766,11 +764,10 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
 		  (tramp-message
 		   v 6 "%s" (mapconcat 'identity (process-command p) " "))
 		  (tramp-set-connection-property p "vector" v)
-		  (tramp-compat-set-process-query-on-exit-flag p nil)
+		  (set-process-query-on-exit-flag p nil)
 		  (tramp-process-actions p v nil tramp-smb-actions-get-acl)
 		  (when (> (point-max) (point-min))
-		    (tramp-compat-funcall
-		     'substring-no-properties (buffer-string)))))
+		    (substring-no-properties (buffer-string)))))
 
 	    ;; Reset the transfer process properties.
 	    (tramp-set-connection-property v "process-name" nil)
@@ -910,17 +907,17 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
   "Like `file-name-all-completions' for Tramp files."
   (all-completions
    filename
-   (with-parsed-tramp-file-name directory nil
+   (with-parsed-tramp-file-name (expand-file-name directory) nil
      (with-tramp-file-property v localname "file-name-all-completions"
        (save-match-data
-	 (let ((entries (tramp-smb-get-file-entries directory)))
-	   (mapcar
-	    (lambda (x)
-	      (list
-	       (if (string-match "d" (nth 1 x))
-		   (file-name-as-directory (nth 0 x))
-		 (nth 0 x))))
-	    entries)))))))
+	 (delete-dups
+	  (mapcar
+	   (lambda (x)
+	     (list
+	      (if (string-match "d" (nth 1 x))
+		  (file-name-as-directory (nth 0 x))
+		(nth 0 x))))
+	   (tramp-smb-get-file-entries directory))))))))
 
 (defun tramp-smb-handle-file-writable-p (filename)
   "Like `file-writable-p' for Tramp files."
@@ -1069,9 +1066,7 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
 	  (tramp-smb-send-command
 	   v
 	   (if (tramp-smb-get-cifs-capabilities v)
-	       (format
-		"posix_mkdir \"%s\" %s"
-		file (tramp-compat-decimal-to-octal (default-file-modes)))
+	       (format "posix_mkdir \"%s\" %o" file (default-file-modes))
 	     (format "mkdir \"%s\"" file)))
 	  ;; We must also flush the cache of the directory, because
 	  ;; `file-attributes' reads the values from there.
@@ -1241,12 +1236,7 @@ target of the symlink differ."
       (unless outbuf
 	(kill-buffer (tramp-get-connection-property v "process-buffer" nil)))
 
-      ;; `process-file-side-effects' has been introduced with GNU
-      ;; Emacs 23.2.  If set to nil, no remote file will be changed
-      ;; by `program'.  If it doesn't exist, we assume its default
-      ;; value t.
-      (unless (and (boundp 'process-file-side-effects)
-		   (not (symbol-value 'process-file-side-effects)))
+      (unless process-file-side-effects
 	(tramp-flush-directory-property v ""))
 
       ;; Return exit status.
@@ -1297,9 +1287,10 @@ target of the symlink differ."
 	      (tramp-error v2 'file-error "Cannot rename `%s'" filename))))
 
       ;; We must rename via copy.
-      (tramp-compat-copy-file filename newname ok-if-already-exists t t t)
+      (copy-file
+       filename newname ok-if-already-exists 'keep-time 'preserve-uid-gid)
       (if (file-directory-p filename)
-	  (tramp-compat-delete-directory filename 'recursive)
+	  (delete-directory filename 'recursive)
 	(delete-file filename)))))
 
 (defun tramp-smb-action-set-acl (proc vec)
@@ -1326,10 +1317,10 @@ target of the symlink differ."
 	       (domain    (tramp-file-name-domain v))
 	       (port      (tramp-file-name-port v))
 	       (share     (tramp-smb-get-share v))
-	       (localname (tramp-compat-replace-regexp-in-string
+	       (localname (replace-regexp-in-string
 			   "\\\\" "/" (tramp-smb-get-localname v)))
 	       (args      (list (concat "//" real-host "/" share) "-E" "-S"
-				(tramp-compat-replace-regexp-in-string
+				(replace-regexp-in-string
 				 "\n" "," acl-string))))
 
 	  (if (not (zerop (length real-user)))
@@ -1365,7 +1356,7 @@ target of the symlink differ."
 		  (tramp-message
 		   v 6 "%s" (mapconcat 'identity (process-command p) " "))
 		  (tramp-set-connection-property p "vector" v)
-		  (tramp-compat-set-process-query-on-exit-flag p nil)
+		  (set-process-query-on-exit-flag p nil)
 		  (tramp-process-actions p v nil tramp-smb-actions-set-acl)
 		  (goto-char (point-max))
 		  (unless (re-search-backward "tramp_exit_status [0-9]+" nil t)
@@ -1388,9 +1379,7 @@ target of the symlink differ."
     (when (tramp-smb-get-cifs-capabilities v)
       (tramp-flush-file-property v localname)
       (unless (tramp-smb-send-command
-	       v (format "chmod \"%s\" %s"
-			 (tramp-smb-get-localname v)
-			 (tramp-compat-decimal-to-octal mode)))
+	       v (format "chmod \"%s\" %o" (tramp-smb-get-localname v) mode))
 	(tramp-error
 	 v 'file-error "Error while changing file's mode %s" filename)))))
 
@@ -1400,16 +1389,18 @@ target of the symlink differ."
 (defun tramp-smb-handle-start-file-process (name buffer program &rest args)
   "Like `start-file-process' for Tramp files."
   (with-parsed-tramp-file-name default-directory nil
-    (let ((command (mapconcat 'identity (cons program args) " "))
-	  (bmp (and (buffer-live-p buffer) (buffer-modified-p buffer)))
-	  (name1 name)
-	  (i 0))
+    (let* ((buffer
+	    (if buffer
+		(get-buffer-create buffer)
+	      ;; BUFFER can be nil.  We use a temporary buffer.
+	      (generate-new-buffer tramp-temp-buffer-name)))
+	   (command (mapconcat 'identity (cons program args) " "))
+	   (bmp (and (buffer-live-p buffer) (buffer-modified-p buffer)))
+	   (name1 name)
+	   (i 0))
       (unwind-protect
 	  (save-excursion
 	    (save-restriction
-	      (unless buffer
-		;; BUFFER can be nil.  We use a temporary buffer.
-		(setq buffer (generate-new-buffer tramp-temp-buffer-name)))
 	      (while (get-process name1)
 		;; NAME must be unique as process name.
 		(setq i (1+ i)
@@ -1461,9 +1452,7 @@ errors for shares like \"C$/\", which are common in Microsoft Windows."
   "Like `write-region' for Tramp files."
   (setq filename (expand-file-name filename))
   (with-parsed-tramp-file-name filename nil
-    ;; XEmacs takes a coding system as the seventh argument, not `confirm'.
-    (when (and (not (featurep 'xemacs))
-	       confirm (file-exists-p filename))
+    (when (and confirm (file-exists-p filename))
       (unless (y-or-n-p (format "File %s exists; overwrite anyway? "
 				filename))
 	(tramp-error v 'file-error "File not overwritten")))
@@ -1576,10 +1565,6 @@ Result is a list of (LOCALNAME MODE SIZE MONTH DAY TIME YEAR)."
 	  ;; Add directory itself.
 	  (push '("" "drwxrwxrwx" 0 (0 0)) res)
 
-	  ;; There's a very strange error (debugged with XEmacs 21.4.14)
-	  ;; If there's no short delay, it returns nil.  No idea about.
-	  (when (featurep 'xemacs) (sleep-for 0.01))
-
 	  ;; Return entries.
 	  (delq nil res))))))
 
@@ -1739,7 +1724,7 @@ Result is the list (LOCALNAME MODE SIZE MTIME)."
 		(member
 		 "pathnames"
 		 (split-string
-		  (buffer-substring (point) (point-at-eol)) nil t)))))))))
+		  (buffer-substring (point) (point-at-eol)) nil 'omit)))))))))
 
 (defun tramp-smb-get-stat-capability (vec)
   "Check, whether the SMB server supports the STAT command."
@@ -1879,7 +1864,7 @@ If ARGUMENT is non-nil, use it as argument for
 	      (tramp-message
 	       vec 6 "%s" (mapconcat 'identity (process-command p) " "))
 	      (tramp-set-connection-property p "vector" vec)
-	      (tramp-compat-set-process-query-on-exit-flag p nil)
+	      (set-process-query-on-exit-flag p nil)
 
 	      ;; Set variables for computing the prompt for reading password.
 	      (setq tramp-current-method tramp-smb-method
@@ -1915,7 +1900,10 @@ If ARGUMENT is non-nil, use it as argument for
 		    ;; character by character; if we send the string
 		    ;; at once, it is read painfully slow.
 		    (tramp-set-connection-property p "smb-share" share)
-		    (tramp-set-connection-property p "chunksize" 1))
+		    (tramp-set-connection-property p "chunksize" 1)
+
+		    ;; Mark it as connected.
+		    (tramp-set-connection-property p "connected" t))
 
 		;; Check for the error reason.  If it was due to wrong
 		;; password, reestablish the connection.  We cannot
